<?php


namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Routing\Exception\InvalidParameterException;

/**
 * Class Question
 * @ORM\Entity(repositoryClass="App\Repository\QuestionRepository")
 * @ORM\Table(name="question")
 * @package App\Entity
 */
class Question
{
    const QSTYPE_SINGLE_CHOICE = "scq";
    const QSTYPE_MULTIPLE_CHOICE = "mcq";
    const QSTYPE_MULTIPLE_CHOICE_RANDOM_WRONGS = "mcqrw";

    const QCTYPE_DEFINITION = "definition";
    const QCTYPE_TARGET = "target";

    const STRUCTURE_TYPES = [
        self::QSTYPE_SINGLE_CHOICE,
        self::QSTYPE_MULTIPLE_CHOICE,
        self::QSTYPE_MULTIPLE_CHOICE_RANDOM_WRONGS
    ];

    const CONTENT_TYPES = [
        self::QCTYPE_DEFINITION,
        self::QCTYPE_TARGET
    ];

    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     */
    protected int $id;

    /**
     * @ORM\Column(nullable=false)
     * @ORM\ManyToOne(fetch="EAGER", inversedBy="questions", targetEntity="App\Entity\Topic")
     */
    protected Topic $topic;

    /**
     * @ORM\Column(columnDefinition="VARCHAR(1024)", length=1024, nullable=false)
     */
    protected string $label;

    /**
     * @ORM\Column(columnDefinition="VARCHAR(10)", name="structure_type", length=10, nullable=false)
     */
    protected string $structureType;

    /**
     * @ORM\Column(columnDefinition="VARCHAR(20)", name="content_type", length=20, nullable=false)
     */
    protected string $contentType;

    /**
     * @ORM\OneToMany(
     *     cascade={"persist", "remove"},
     *     fetch="EAGER",
     *     mappedBy="question",
     *     orphanRemoval=true,
     *     targetEntity="App\Entity\Answer"
     * )
     */
    protected ArrayCollection $answers;

    public function __construct()
    {
        $this->answers = new ArrayCollection();
    }

    public function setStructureType(string $structureType): self
    {
        if (!in_array($structureType, self::STRUCTURE_TYPES)) {
            throw new InvalidParameterException(sprintf(
                "Cannot assign an invalid question structure type - use one of %s.",
                implode(', ', self::STRUCTURE_TYPES)
            ));
        }

        $this->structureType = $structureType;
        return $this;
    }

    public function setContent(string $contentType): self
    {
        if (!in_array($contentType, self::CONTENT_TYPES)) {
            throw new InvalidParameterException(sprintf(
                "Cannot assign an invalid question content type - use one of%s.",
                implode(', ', self::CONTENT_TYPES)
            ));
        }

        $this->contentType = $contentType;
        return $this;
    }

    public function addAnswer(Answer $answer) : self
    {
        $this->answers->add($answer);
        return $this;
    }

    public function getAnswers(): ArrayCollection
    {
        return $this->answers;
    }

    public function getLabel(): ?string
    {
        return $this->label;
    }

    public function getStructureType() : ?string
    {
        return $this->structureType;
    }

    public function getContentType() : ?string
    {
        return $this->contentType;
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function isSingleChoice(): bool
    {
        return self::QSTYPE_SINGLE_CHOICE === $this->structureType;
    }

    public function isMultipleChoiceRandomWrongs(): bool
    {
        return self::QSTYPE_MULTIPLE_CHOICE_RANDOM_WRONGS === $this->structureType;
    }
}